@@ -7,12 +7,13 @@
 #ifndef CONF_H_
 #define CONF_H_
 
-#ifdef _XOPEN_SOURCE
+#if (defined _XOPEN_SOURCE || defined __APPLE__)
 
 // *at support, such as openat, utimensat, etc (see man 2 openat)
 #include <fcntl.h>
 #include <sys/stat.h>
-#if !defined (DISABLE_AT) && (_XOPEN_SOURCE >= 700 && _POSIX_C_SOURCE >= 200809L) \
+#if !defined (DISABLE_AT) && \
+    ((_XOPEN_SOURCE >= 700 && _POSIX_C_SOURCE >= 200809L) || (defined __APPLE__)) \
 	&& defined (AT_SYMLINK_NOFOLLOW)
 	#define UNIONFS_HAVE_AT
 #endif
@@ -36,4 +37,3 @@
 
 
 #endif // CONF_H_
-
