@@ -27,96 +27,28 @@
 #include "usyslog.h"
 
 
-/**
- * Actually create the directory here.
- */
-static int do_create(const char *path, int nbranch_ro, int nbranch_rw) {
-	DBG("%s\n", path);
-
-	char dirp[PATHLEN_MAX]; // dir path to create
-	sprintf(dirp, "%s%s", uopt.branches[nbranch_rw].path, path);
-
-	struct stat buf;
-	int res = stat(dirp, &buf);
-	if (res != -1) RETURN(0); // already exists
-
-	if (nbranch_ro == nbranch_rw) {
-		// special case nbranch_ro = nbranch_rw, this is if we a create
-		// unionfs meta directories, so not directly on cow operations
-		buf.st_mode = S_IRWXU | S_IRWXG;
-	} else {
-		// data from the ro-branch
-		char o_dirp[PATHLEN_MAX]; // the pathname we want to copy
-		sprintf(o_dirp, "%s%s", uopt.branches[nbranch_ro].path, path);
-		res = stat(o_dirp, &buf);
-		if (res == -1) RETURN(1); // lower level branch removed in the mean time?
-	}
-
-	res = mkdir(dirp, buf.st_mode);
-	if (res == -1) {
-		USYSLOG(LOG_DAEMON, "Creating %s failed: \n", dirp);
-		RETURN(1);
-	}
-
-	if (nbranch_ro == nbranch_rw) RETURN(0); // the special case again
-
-	if (setfile(dirp, &buf)) RETURN(1); // directory already removed by another process?
-
-	// TODO: time, but its values are modified by the next dir/file creation steps?
-
-	RETURN(0);
-}
-
 /**
  * l_nbranch (lower nbranch than nbranch) is write protected, create the dir path on
  * nbranch for an other COW operation.
  */
-int path_create(const char *path, int nbranch_ro, int nbranch_rw) {
+int path_create_cow(const char *path, int nbranch_ro, int nbranch_rw) {
 	DBG("%s\n", path);
 
 	if (!uopt.cow_enabled) RETURN(0);
 
-	char p[PATHLEN_MAX];
-	if (BUILD_PATH(p, uopt.branches[nbranch_rw].path, path)) RETURN(-ENAMETOOLONG);
-
-	struct stat st;
-	if (!stat(p, &st)) {
-		// path does already exists, no need to create it
-		RETURN(0);
-	}
-
-	char *walk = (char *)path;
-
-	// first slashes, e.g. we have path = /dir1/dir2/, will set walk = dir1/dir2/
-	while (*walk == '/') walk++;
-
-	do {
-		// walk over the directory name, walk will now be /dir2
-		while (*walk != '\0' && *walk != '/') walk++;
-
-		// +1 due to \0, which gets added automatically
-		snprintf(p, (walk - path) + 1, "%s", path); // walk - path = strlen(/dir1)
-		int res = do_create(p, nbranch_ro, nbranch_rw);
-		if (res) RETURN(res); // creating the directory failed
-
-		// as above the do loop, walk over the next slashes, walk = dir2/
-		while (*walk == '/') walk++;
-	} while (*walk != '\0');
-
-	RETURN(0);
+	return path_create(path, nbranch_ro, nbranch_rw);
 }
 
 /**
- * Same as  path_create(), but ignore the last segment in path,
+ * Same as  path_create_cow(), but ignore the last segment in path,
  * i.e. it might be a filename.
  */
-int path_create_cutlast(const char *path, int nbranch_ro, int nbranch_rw) {
+int path_create_cutlast_cow(const char *path, int nbranch_ro, int nbranch_rw) {
 	DBG("%s\n", path);
 
 	char *dname = u_dirname(path);
-	if (dname == NULL)
-		RETURN(-ENOMEM);
-	int ret = path_create(dname, nbranch_ro, nbranch_rw);
+	if (dname == NULL) RETURN(-ENOMEM);
+	int ret = path_create_cow(dname, nbranch_ro, nbranch_rw);
 	free(dname);
 
 	RETURN(ret);
@@ -125,17 +57,20 @@ int path_create_cutlast(const char *path, int nbranch_ro, int nbranch_rw) {
 /**
  * initiate the cow-copy action
  */
-int cow_cp(const char *path, int branch_ro, int branch_rw, bool copy_dir) {
+int cow_cp(const char *path, int branch_ro, int branch_rw, bool recursive) {
 	DBG("%s\n", path);
 
 	// create the path to the file
-	path_create_cutlast(path, branch_ro, branch_rw);
+	int res = path_create_cutlast_cow(path, branch_ro, branch_rw);
+	if (res != 0) RETURN(res);
 
 	char from[PATHLEN_MAX], to[PATHLEN_MAX];
-	if (BUILD_PATH(from, uopt.branches[branch_ro].path, path))
+	if (BUILD_PATH(from, uopt.branches[branch_ro].path, path)) {
 		RETURN(-ENAMETOOLONG);
-	if (BUILD_PATH(to, uopt.branches[branch_rw].path, path))
+	}
+	if (BUILD_PATH(to, uopt.branches[branch_rw].path, path)) {
 		RETURN(-ENAMETOOLONG);
+	}
 
 	setlocale(LC_ALL, "");
 
@@ -154,16 +89,15 @@ int cow_cp(const char *path, int branch_ro, int branch_rw, bool copy_dir) {
 	lstat(cow.from_path, &buf);
 	cow.stat = &buf;
 
-	int res;
 	switch (buf.st_mode & S_IFMT) {
 		case S_IFLNK:
 			res = copy_link(&cow);
 			break;
 		case S_IFDIR:
-			if (copy_dir) {
+			if (recursive) {
 				res = copy_directory(path, branch_ro, branch_rw);
 			} else {
-				res = path_create(path, branch_ro, branch_rw);
+				res = path_create_cow(path, branch_ro, branch_rw);
 			}
 			break;
 		case S_IFBLK:
@@ -190,7 +124,7 @@ int copy_directory(const char *path, int branch_ro, int branch_rw) {
 	DBG("%s\n", path);
 
 	/* create the directory on the destination branch */
-	int res = path_create(path, branch_ro, branch_rw);
+	int res = path_create_cow(path, branch_ro, branch_rw);
 	if (res != 0) {
 		RETURN(res);
 	}
@@ -211,6 +145,45 @@ int copy_directory(const char *path, int branch_ro, int branch_rw) {
 			res = 1;
 			break;
 		}
+
+		// Generally if the target file already exists, we should not copy
+		// anything. Directories are a special case as their contents may still
+		// need to be merged.
+		bool is_dir = false;
+		if (branch_contains_path(branch_rw, member, &is_dir) &&
+			(!is_dir || (branch_contains_path(branch_ro, member, &is_dir) && !is_dir))) {
+			// File already exists in target and either source or target is not
+			// a directory, skip it
+			DBG("file %s copy skipped, exists in target\n", member);
+			continue;
+		}
+
+		// If source file is hidden by a higher branch, we should not copy
+		// anything. We do this by iterating through all higher branches and
+		// checking if they have a whiteout. This is somewhat inefficient, but
+		// it is the only simple way to handle this case. A more complex
+		// solution would be to collect whiteouts to a hashtable, as has been
+		// done in readdir.
+		bool skip = false;
+		for (int i = 0; i < branch_ro; i++) {
+			// Assuming that only rw branches can have whiteouts
+			if (uopt.branches[i].rw) {
+				int hidden = path_hidden(member, i);
+				if (hidden > 0) {
+					// File was hidden, skip it
+					DBG("file %s copy skipped, hidden by layer %i\n", member, i);
+					skip = true;
+					break;
+				} else if (hidden < 0) {
+					// error in path_hidden
+					res = hidden;
+					break;
+				}
+			}
+		}
+		if (res != 0) break;
+		if (skip) continue;
+
 		res = cow_cp(member, branch_ro, branch_rw, true);
 		if (res != 0) break;
 	}
