@@ -27,6 +27,8 @@ int hide_dir(const char *path, int branch_rw);
 filetype_t path_is_dir (const char *path);
 int maybe_whiteout(const char *path, int branch_rw, enum whiteout mode);
 int set_owner(const char *path);
+int path_create(const char *path, int nbranch_ro, int nbranch_rw);
+int path_create_cutlast(const char *path, int nbranch_ro, int nbranch_rw);
 
 
 #endif
