@@ -17,6 +17,21 @@
 #define FUSE_META_FILE ".fuse_hidden"
 #define FUSE_META_LENGTH 12
 
+#ifndef IOCPARM_MASK
+#define IOCPARM_MASK      0x1FFF
+#endif
+#ifndef IOCPARM_LEN
+#define IOCPARM_LEN(a)    (((a) >> 16) & IOCPARM_MASK)
+#endif
+
+#ifndef _IOC_SIZE
+#ifdef IOCPARM_LEN
+#define _IOC_SIZE(x) IOCPARM_LEN(x)
+#else
+#error "No mechanism for determining ioctl length found."
+#endif
+#endif
+
 // file access protection mask
 #define S_PROT_MASK (S_ISUID| S_ISGID | S_ISVTX | S_IRWXU | S_IRWXG | S_IRWXO)
 
@@ -27,4 +42,6 @@ typedef struct {
 	unsigned char rw;	 // the writable flag
 } branch_entry_t;
 
+extern struct fuse_operations unionfs_oper;
+
 #endif
