@@ -2,15 +2,15 @@
  * License: BSD-style license
  * Copyright: Bernd Schubert <bernd.schubert@fastmail.fm>
  *
- * Details: 
+ * Details:
  *   Log files might be located on our own filesystem. If we then want to log
  *   a message to syslog, we would need to log to ourself, which easily ends up
- *   in a deadlock. Initializing openlog() using the flags 
+ *   in a deadlock. Initializing openlog() using the flags
  *   LOG_NDELAY | LOG_NOWAIT should prevent that, but real live has shown that
- *   this does not work reliable and systems 'crashed' just because we 
- *   tried to log a harmless message. 
- *   So this file introduces a syslog thread and a syslog buffer. usyslog() 
- *   calls write without a risk to deadlock into the syslog buffer (chained 
+ *   this does not work reliable and systems 'crashed' just because we
+ *   tried to log a harmless message.
+ *   So this file introduces a syslog thread and a syslog buffer. usyslog()
+ *   calls write without a risk to deadlock into the syslog buffer (chained
  *   list) and then the seperate syslog_thread call syslog(). That way our
  *   our filesystem thread(s) cannot stall from syslog() calls.
  *
@@ -50,7 +50,7 @@ static pthread_mutex_t sleep_mutex;     // locks cond_message
 static pthread_cond_t cond_message;		// used to wake up the syslog thread
 
 // Only used for debugging, protected by list_lock
-static int free_entries;  
+static int free_entries;
 static int used_entries = 0;
 
 //#define USYSLOG_DEBUG
@@ -59,7 +59,7 @@ static int used_entries = 0;
 static void verify_lists()
 {
 	pthread_mutex_lock(&list_lock);
-	
+
 	ulogs_t *entry = free_head;
 	int free_count = -1;
 	bool first_free = true;
@@ -71,10 +71,10 @@ static void verify_lists()
 			free_count++;
 		entry = entry->next;
 	}
-	if (free_count != free_entries && free_entries != 0) 
+	if (free_count != free_entries && free_entries != 0)
 		DBG("usyslog list error detected: number of free entries inconsistent!"
 		     " %d vs. %d", free_count, free_entries);
-		     
+
 	entry = used_head;
 	int used_count = -1;
 	bool first_used = true;
@@ -86,11 +86,11 @@ static void verify_lists()
 			used_count++;
 		entry = entry->next;
 	}
-	if (used_count != used_entries && used_entries != 0) 
+	if (used_count != used_entries && used_entries != 0)
 		DBG("usyslog list error detected: number of used entries inconsistent!"
-		     " (used: %d vs. %d) (free: %d vs. %d) \n", 
+		     " (used: %d vs. %d) (free: %d vs. %d) \n",
 		     used_count, used_entries, free_count, free_entries);
-		     
+
 	pthread_mutex_unlock(&list_lock);
 }
 #else
@@ -111,8 +111,8 @@ static void do_syslog(void)
 		pthread_mutex_t *entry_lock = &log_entry->lock;
 		int res = pthread_mutex_trylock(entry_lock);
 		if (res) {
-			if (res != EBUSY) 
-				DBG("Entirely unexpected locking error %s\n", 
+			if (res != EBUSY)
+				DBG("Entirely unexpected locking error %s\n",
 				    strerror(res));
 			// If something goes wrong with the log_entry we do not
 			// block the critical list_lock forever!
@@ -124,7 +124,7 @@ static void do_syslog(void)
 			continue;
 		}
 		pthread_mutex_unlock(&list_lock);
-		
+
 		// This syslog call and so this lock might block, so be
 		// carefull on using locks! The filesystem IO thread
 		// *MUST* only try to lock it using pthread_mutex_trylock()
@@ -134,42 +134,42 @@ static void do_syslog(void)
 		// NOTE: The list is only locked now, after syslog() succeeded!
 		pthread_mutex_lock(&list_lock);
 		ulogs_t *next_entry = log_entry->next; // just to save the pointer
-		
+
 		used_head = log_entry->next;
 		if (!used_head)
 			used_bottom = NULL; // no used entries left
-		
+
 		if (free_bottom)
 			free_bottom->next = log_entry;
 		free_bottom = log_entry;
-		free_bottom->next = NULL; 
-		
+		free_bottom->next = NULL;
+
 		if (!free_head)
 			free_head = log_entry;
-			
+
 		free_entries++;
 		used_entries--;
-			
+
 		pthread_mutex_unlock(&list_lock); // unlock ist ASAP
-		
+
 		log_entry = next_entry;
 		pthread_mutex_unlock(entry_lock);
 	}
-	
+
 	verify_lists();
 }
 
 /**
- * syslog backgroung thread that tries to to empty the syslog buffer
+ * syslog background thread that tries to to empty the syslog buffer
  */
 static void * syslog_thread(void *arg)
 {
 	// FIXME: What is a better way to prevent a compiler warning about
 	//        unused variable 'arg'
 	int tinfo = *((int *) arg);
-	if (tinfo == 0 && tinfo == 1)
-		printf("Starting thread %d", tinfo);
-
+	if (tinfo == 0) {
+		//printf("Starting thread %d", tinfo);
+	}
 
 	pthread_mutex_lock(&sleep_mutex);
 	while (1) {
@@ -194,7 +194,7 @@ void usyslog(int priority, const char *format, ...)
 	// Lock the entire list first, which means the syslog thread MUST NOT
 	// lock it if there is any chance it might be locked forever.
 	pthread_mutex_lock(&list_lock);
-	
+
 	// Some sanity checks. If we fail here, we will leak a log entry,
 	// but will not lock up.
 
@@ -203,7 +203,7 @@ void usyslog(int priority, const char *format, ...)
 		pthread_mutex_unlock(&list_lock);
 		return;
 	}
-	
+
 	log = free_head;
 	free_head = log->next;
 
@@ -234,22 +234,21 @@ void usyslog(int priority, const char *format, ...)
 		used_bottom->next = log;
 		used_bottom = log;
 	}
-	
+
 	if (log->next) {
 		// from free_list to end of used_list, so next is NULL now
-		log->next = NULL; 
+		log->next = NULL;
 	} else {
 		// so the last entry in free_list
 		free_bottom = NULL;
 	}
-	
-	
+
 	free_entries--;
 	used_entries++;
-		
+
 	// Everything below is log entry related, so we can free the list_lock
 	pthread_mutex_unlock(&list_lock);
-	
+
 	va_list ap;
 	va_start(ap, format);
 	vsnprintf(log->message, MAX_MSG_SIZE, format, ap);
@@ -258,7 +257,6 @@ void usyslog(int priority, const char *format, ...)
 
 	pthread_mutex_unlock(&log->lock);
 
-
 	pthread_mutex_lock(&sleep_mutex);
 
 	pthread_cond_signal(&cond_message); // wake up the syslog thread
@@ -291,12 +289,12 @@ void init_syslog(void)
 			fprintf(stderr, "\nLog initialization failed: %s\n", strerror(errno));
 			fprintf(stderr, "Aborting!\n");
 			// Still initialazation phase, we can abort.
-			exit (1);	
+			exit(1);
 		}
 
 		log->used = false;
 		pthread_mutex_init(&log->lock, NULL);
-		
+
 		if (last) {
 			last->next = log;
 		} else {
@@ -307,7 +305,7 @@ void init_syslog(void)
 	}
 	last->next = NULL;
 	free_bottom = last;
-	
+
 	free_entries = MAX_SYSLOG_MESSAGES;
 
 	pthread_attr_init(&attr);
